__all__ = ['StringAttributeType', 'StringSubtype',
           'EnumAttributeType', 'EnumItem', 'EnumSubtype',
           'NullAttributeType',
           'BooleanAttributeType',
           'IntegerAttributeType', 'IntegerSubtype']

import aenum
from gm_meta_plugin.attribute import Attribute, AttributeType
from gm_meta_plugin.errors import InvalidAttributeError


class StringSubtype(aenum.Enum):
    NONE = "NONE"
    FRAME_RANGES = "FRAME_RANGES"
    PATH = "PATH"


class StringAttributeType(Attribute):

    def __init__(self,
                 key,
                 display_name,
                 description,
                 subtype_kwargs,
                 default_value = "",
                 subtype = StringSubtype.NONE.value,
                 placeholder=None,
                 max_length = None,
                 min_length = None):

        Attribute.__init__(self, key, display_name, description, AttributeType.STRING, subtype_kwargs)

        if default_value is None:
            default_value = ""

        if subtype is None:
            subtype = StringSubtype.NONE.value

        self._default_value = default_value
        self._subtype = subtype
        self._placeholder = placeholder
        self._max_length = max_length
        self._min_length = min_length

    def get_default_value(self):
        return self._default_value

    def get_subtype(self):
        return self._subtype

    def get_placeholder(self):
        return self._placeholder

    def get_max_length(self):
        return self._max_length

    def get_min_length(self):
        return self._min_length

    def validate_value(self, value):
        if value is None:
            raise InvalidAttributeError("value must not be None")

        if type(value) != str:
            raise InvalidAttributeError("value must be string type")

        if self._max_length is not None and len(value) > self._max_length:
            raise InvalidAttributeError("value be be a maximum of " + unicode(self._max_length) + " characters long")

        if self._min_length is not None and len(value) < self._min_length:
            raise InvalidAttributeError("value be be a minimum of " + unicode(self._min_length) + " characters long")


class EnumItem:

    def __init__(self, key, display_name, description):
        self._key = key
        self._display_name = display_name
        self._description = description

    def get_key(self):
        return self._key

    def get_display_name(self):
        return self._display_name

    def get_description(self):
        return self._description


class EnumSubtype(aenum.Enum):
    NONE = "NONE"
    APP_INFERRED = "APP_INFERRED"
    PRODUCT_VERSIONS = "PRODUCT_VERSIONS"
    MACHINE_TYPE = "MACHINE_TYPE"


class EnumAttributeType(Attribute):

    DYNAMIC_ENUM_SUBTYPES = [EnumSubtype.PRODUCT_VERSIONS.value,
                             EnumSubtype.MACHINE_TYPE.value,
                             EnumSubtype.APP_INFERRED.value]

    def __init__(self,
                 key,
                 display_name,
                 description,
                 subtype_kwargs,
                 items,
                 default_value = None,
                 subtype = EnumSubtype.NONE.value):

        Attribute.__init__(self, key, display_name, description, AttributeType.ENUM, subtype_kwargs)

        if len(items) <= 0 and subtype not in self.DYNAMIC_ENUM_SUBTYPES:
            raise ValueError("Enum '" + key + "' must have at least one possible value")

        self._items = items
        self._subtype = subtype

        if subtype in self.DYNAMIC_ENUM_SUBTYPES:
            # items and default value do not matter for PRODUCT_VERSIONS subtype
            self._default_value = None
        elif default_value is None:
            # by default use the first enum item as a default value if no default value was provided
            self._default_value = items[0].get_key()
        else:
            self._default_value = default_value

    def get_default_value(self):
        return self._default_value

    def get_default_enum_item(self):
        for item in self._items:
            if item.get_key() == self.get_default_value():
                return item
        return None

    def get_subtype(self):
        return self._subtype

    def get_items(self):
        return self._items

    def validate_value(self, value):
        if value is None:
            raise InvalidAttributeError("value must not be None")

        if type(value) != str:
            raise InvalidAttributeError("value must be string type")

        # don't compare items for dynamic enums
        if self.get_subtype() in self.DYNAMIC_ENUM_SUBTYPES:
            return

        for item in self.get_items():
            if item.get_key() == value:
                return
        else:
            raise InvalidAttributeError("value must be one of the possible enum values")


class NullAttributeType(Attribute):

    def __init__(self, key, display_name, description, subtype_kwargs):
        Attribute.__init__(self, key, display_name, description, AttributeType.NULL, subtype_kwargs)
        self._default_value = None

    def get_default_value(self):
        return self._default_value

    def validate_value(self, value):
        if value is not None:
            raise InvalidAttributeError("value must be None")


class BooleanAttributeType(Attribute):

    def __init__(self,
                 key,
                 display_name,
                 description,
                 subtype_kwargs,
                 default_value = True):

        Attribute.__init__(self, key, display_name, description, AttributeType.BOOLEAN, subtype_kwargs)

        if default_value is None:
            default_value = False

        self._default_value = default_value

    def get_default_value(self):
        return self._default_value

    def validate_value(self, value):
        if value is None:
            raise InvalidAttributeError("value must not be None")

        if type(value) != bool:
            raise InvalidAttributeError("value must be bool type")


class IntegerSubtype(aenum.Enum):
    NONE = "NONE"
    INSTANCES = "INSTANCES"


class IntegerAttributeType(Attribute):

    def __init__(self,
                 key,
                 display_name,
                 description,
                 subtype_kwargs,
                 default_value = 0,
                 subtype = IntegerSubtype.NONE.value):

        Attribute.__init__(self, key, display_name, description, AttributeType.INTEGER, subtype_kwargs)

        if default_value is None:
            default_value = 0

        self._default_value = default_value
        self._subtype = subtype

    def get_default_value(self):
        return self._default_value

    def get_subtype(self):
        return self._subtype

    def validate_value(self, value):
        if value is None:
            raise InvalidAttributeError("value must not be None")

        if type(value) != int:
            raise InvalidAttributeError("value must be int type")
