__all__ = 'LoggingCoordinator'

import io
import os
import queue

from .logger import Logger


class LoggingCoordinator:
    """
    A class for creating loggers and storing their messages in a list that the ui can draw in a thread safe manner.
    """

    def __init__(self, plugin, log_history_container):
        self._log_history_container = log_history_container
        self._multi_thread_counter = 0
        self._queue = queue.Queue()
        self._plugin = plugin

    def get_plugin(self):
        return self._plugin

    def is_thread_safe_mode_enabled(self):
        return self._multi_thread_counter > 0

    def set_thread_safe_mode(self, enabled):
        if enabled:
            self._multi_thread_counter += 1
        else:
            self._multi_thread_counter -= 1

            if self._multi_thread_counter < 0:
                raise ValueError("Muti-thread counter should never be less than 1")

    def add_thread_safe_logging_lock(self):
        self.set_thread_safe_mode(True)

    def remove_thread_safe_logging_lock(self):
        self.set_thread_safe_mode(False)

    def get_logger(self, name=None):
        return Logger(name, self)

    def get_log_history_container(self):
        return self._log_history_container

    def serialize_logs(self):
        sys_info = self.get_plugin().get_plugin_utils().get_sys_info()

        output = ''

        for log_item in self.get_log_history_container().get_all():
            output += log_item.serialize() + os.linesep

        return sys_info + os.linesep + output

    def copy_logs_to_clipboard(self):
        plugin_utils = self.get_plugin().get_plugin_utils()
        plugin_utils.copy_to_clipboard(self.serialize_logs())

    def save_logs_to_file(self, file_path):
        with io.open(file_path, 'w', encoding='utf-8') as f:
            f.write(self.serialize_logs())
            f.close()

    def queue(self, log_item):
        """
        Add the log_item to the queue if operating in multi-threaded mode, otherwise append straight to the log history
        container.
        """

        self._queue.put(log_item)

        if not self.is_thread_safe_mode_enabled():
            self.flush()

    def flush(self):
        """ Call this in the main thread. """

        while True:
            try:
                logged_item = self._queue.get(block=False)
                self._log_history_container.append(logged_item)
            except queue.Empty:
                return

    def remove_lock_and_flush(self):
        self.remove_thread_safe_logging_lock()
        self.flush()
