__all__ = "ExceptionPopup"

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

from gm_qt_plugin import qt_constants
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor


class ExceptionPopup(QtWidgets.QDialog, QtPluginAccessor):
    """
    This popup is used when an unexpected exception is encountered and the user must be informed that something went
    very wrong.
    """

    INITIAL_WINDOW_WIDTH = 900
    INITIAL_WINDOW_HEIGHT = 500

    def __init__(self, plugin, stack_trace, window_title=None, parent=None):

        if window_title is None:
            window_title = u"Exception"

        if parent is None:
            parent = plugin.get_user_interface().get_main_window()

        QtWidgets.QDialog.__init__(self, parent)
        QtPluginAccessor.__init__(self, plugin)

        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)

        self.setWindowModality(QtCore.Qt.ApplicationModal)

        # apply style sheet
        plugin.get_user_interface().set_style_sheet(self)

        # Remove the window help button that only appears on Windows OS
        self.setWindowFlags(self.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint)

        # Set the window title to the plugin name
        self.setWindowTitle(window_title)

        # resize the window to it's initial size
        self.resize(ExceptionPopup.INITIAL_WINDOW_WIDTH, ExceptionPopup.INITIAL_WINDOW_HEIGHT)

        # set fixed dimensions
        self.setFixedWidth(ExceptionPopup.INITIAL_WINDOW_WIDTH)
        self.setFixedHeight(ExceptionPopup.INITIAL_WINDOW_HEIGHT)

        self.setWindowIcon(plugin.get_user_interface().get_icon())

        """ Create Widgets """
        self._icon = QtWidgets.QLabel(":(")
        self._icon.setProperty(qt_constants.STYLE_PROPS.H2, True)

        self._header = QtWidgets.QLabel(u"Unexpected Error")
        self._header.setProperty(qt_constants.STYLE_PROPS.H2, True)

        theme = plugin.get_user_interface().get_theme()
        html = u"""
        <div>
            <div style="color: """ + theme.FOREGROUND_COLOR_3 + u""";">
                Sorry, an unexpected error has occurred
            </div>
            <br/>
            <div>
                Please contact our support team at """ + qt_constants.SUPPORT_EMAIL + u"""
            </div>
        </div>
        """

        self._message = QtWidgets.QLabel(html)
        self._message.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
        self._message.setProperty(qt_constants.STYLE_PROPS.H4, True)

        self._extra_message = QtWidgets.QLabel(u"Below is a trace of the error, please include this in your support"
                                               u" ticked to help us identify the issue.")

        self._stack_trace = QtWidgets.QPlainTextEdit()
        self._stack_trace.setReadOnly(True)
        self._stack_trace.setPlainText(stack_trace)
        self._stack_trace.setWordWrapMode(QtGui.QTextOption.NoWrap)
        self._stack_trace.setProperty(qt_constants.STYLE_PROPS.CONSOLE, True)

        self._close_btn = QtWidgets.QPushButton(u"Close")
        self._close_btn.clicked.connect(self.close)
        self._close_btn.setMaximumWidth(120)
        self._close_btn.setProperty(qt_constants.STYLE_PROPS.BRAND, True)

        self._col_1_widget = QtWidgets.QWidget()
        self._col_2_widget = QtWidgets.QWidget()

        """ Create Layouts """
        self._column_1 = QtWidgets.QVBoxLayout()
        self._column_1.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self._column_1.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)
        self._column_1.addWidget(self._icon)
        self._col_1_widget.setLayout(self._column_1)
        self._col_1_widget.setMaximumWidth(64)

        self._controls_layout = QtWidgets.QHBoxLayout()
        self._controls_layout.setAlignment(QtCore.Qt.AlignRight)
        self._controls_layout.addStretch()
        self._controls_layout.addWidget(self._close_btn)

        self._column_2 = QtWidgets.QVBoxLayout()
        self._column_2.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)
        self._column_2.addWidget(self._header)
        self._column_2.addWidget(self._message)
        self._column_2.addWidget(self._extra_message)
        self._column_2.addSpacing(10)
        self._column_2.addWidget(self._stack_trace)
        self._column_2.addSpacing(10)
        self._column_2.addLayout(self._controls_layout)
        self._col_2_widget.setLayout(self._column_2)

        self._layout = QtWidgets.QHBoxLayout(self)
        self._layout.addWidget(self._col_1_widget)
        self._layout.addWidget(self._col_2_widget)

        # show the popup
        self.exec_()
