__all__ = 'QtUserInterface'

from abc import abstractmethod

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_qt_plugin import qt_constants
from gm_qt_plugin.worker_thread import WorkerThread
from gm_qt_plugin.qt_dialog_available_credits_popup import AvailableCreditsPopup


class QtUserInterface(gm_meta_plugin.UserInterface, QtWidgets.QWidget):

    operation_started = QtCore.Signal()
    operation_finished = QtCore.Signal()

    CUSTOM_FONT_FILES = [
        u"Montserrat/Montserrat-Black.ttf",
        u"Montserrat/Montserrat-BlackItalic.ttf",
        u"Montserrat/Montserrat-Bold.ttf",
        u"Montserrat/Montserrat-BoldItalic.ttf",
        u"Montserrat/Montserrat-ExtraBold.ttf",
        u"Montserrat/Montserrat-ExtraBoldItalic.ttf",
        u"Montserrat/Montserrat-ExtraLight.ttf",
        u"Montserrat/Montserrat-ExtraLightItalic.ttf",
        u"Montserrat/Montserrat-Italic.ttf",
        u"Montserrat/Montserrat-Light.ttf",
        u"Montserrat/Montserrat-LightItalic.ttf",
        u"Montserrat/Montserrat-Medium.ttf",
        u"Montserrat/Montserrat-MediumItalic.ttf",
        u"Montserrat/Montserrat-Regular.ttf",
        u"Montserrat/Montserrat-SemiBold.ttf",
        u"Montserrat/Montserrat-SemiBoldItalic.ttf",
        u"Montserrat/Montserrat-Thin.ttf",
        u"Montserrat/Montserrat-ThinItalic.ttf",
        u"Open_Sans/OpenSans-Bold.ttf",
        u"Open_Sans/OpenSans-BoldItalic.ttf",
        u"Open_Sans/OpenSans-ExtraBold.ttf",
        u"Open_Sans/OpenSans-ExtraBoldItalic.ttf",
        u"Open_Sans/OpenSans-Italic.ttf",
        u"Open_Sans/OpenSans-Light.ttf",
        u"Open_Sans/OpenSans-LightItalic.ttf",
        u"Open_Sans/OpenSans-Regular.ttf",
        u"Open_Sans/OpenSans-SemiBold.ttf",
        u"Open_Sans/OpenSans-SemiBoldItalic.ttf"
    ]

    LOADED_FONTS = []

    def __init__(self, plugin):
        QtWidgets.QWidget.__init__(self, None)

        self._plugin = plugin

        self._auth_email_is_valid = True
        self._auth_email_validity_message = u""

        self._auth_access_key_is_valid = True
        self._auth_access_key_validity_message = u""

        self._auth_user_is_valid = True
        self._auth_user_validity_message = u""

        self._use_dark_theme = True
        self._styled_widgets = []

    def get_main_window(self):
        return self._main_window

    def get_main_switcher_ui(self):
        return self._main_window.get_main_ui_widget().get_main_ui_switcher()

    def set_main_window(self, main_window):
        self._main_window = main_window

    def get_auth_email(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.get_widget().text()

    def set_auth_email(self, email):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.get_widget().setText(email)

    def get_auth_email_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.is_valid()

    def set_auth_email_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.set_validity_flag(is_valid)

    def get_auth_email_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.get_warning_message()

    def set_auth_email_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.set_warning_message(message)

    def get_auth_access_key(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.get_widget().text()

    def set_auth_access_key(self, email):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.get_widget().setText(email)

    def get_auth_access_key_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.is_valid()

    def set_auth_access_key_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.set_validity_flag(is_valid)

    def get_auth_access_key_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.get_warning_message()

    def set_auth_access_key_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.set_warning_message(message)

    def get_user_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.sign_in_btn.is_valid()

    def set_user_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.sign_in_btn.set_validity_flag(is_valid)
        sign_in_widget.sign_in_envoy_btn.set_validity_flag(is_valid)

    def get_user_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.sign_in_btn.get_warning_message()

    def set_user_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.sign_in_btn.set_warning_message(message)
        sign_in_widget.sign_in_envoy_btn.set_warning_message(message)

    def is_running_operation(self):
        status_indicator = self._main_window.get_main_ui_widget().get_header().get_status_indicator()
        return status_indicator.isVisible()

    def set_is_running_operation_flag(self, value):
        status_indicator = self._main_window.get_main_ui_widget().get_header().get_status_indicator()
        status_indicator.setVisible(value)

    def get_running_operation_message(self):
        status_indicator = self._main_window.get_main_ui_widget().get_header().get_status_indicator()
        return status_indicator.get_status_message()

    def set_running_operation_message(self, message):
        status_indicator = self._main_window.get_main_ui_widget().get_header().get_status_indicator()
        status_indicator.set_status_message(message)

    def load_profile(self, user_profile):
        raise NotImplementedError

    def show_log_dates(self):
        raise NotImplementedError

    def set_show_log_dates(self, enabled):
        raise NotImplementedError

    def show_log_times(self):
        raise NotImplementedError

    def set_show_log_times(self, enabled):
        raise NotImplementedError

    def show_logger_names(self):
        raise NotImplementedError

    def set_show_logger_names(self, enabled):
        raise NotImplementedError

    def get_show_hidden_job_preset_attributes(self):
        raise NotImplementedError

    def set_show_hidden_job_preset_attributes(self, value):
        raise NotImplementedError

    def get_worker_thread(self, status_message):

        def _set_running_operation(message):
            self.set_is_running_operation_flag(True)
            self.set_running_operation_message(message)
            self.operation_started.emit()

        def _hide_running_operation_message():
            self.set_is_running_operation_flag(False)
            self.operation_finished.emit()

        self._worker_thread = WorkerThread(self.get_plugin())
        self._worker_thread.started.connect(lambda: _set_running_operation(status_message))
        self._worker_thread.finished.connect(_hide_running_operation_message)

        return self._worker_thread

    def show_available_credits_popup(self):
        plugin = self.get_plugin()
        api_client = plugin.get_api_client()

        # if no user is signed in then exit
        if not api_client.is_user_signed_in():
            return

        self._worker_thread = self.get_worker_thread("Checking Available Credits...")

        ################################################################################################################

        target = api_client.get_available_credits

        args = tuple()

        kwargs = {}

        self._worker_thread.process(target, args, kwargs, self.show_available_credits_popup_slot)

    def show_available_credits_popup_slot(self, result):

        if type(result) is float:
            main_window = self.get_main_window()

            main_window._available_credits_popup = AvailableCreditsPopup(self.get_plugin(),
                                                                         credits=result,
                                                                         parent=self.get_main_window())
            main_window._available_credits_popup.setFixedWidth(250)
            main_window._available_credits_popup.setFixedHeight(60)

            pos = main_window.pos()
            pos.setX((pos.x() + main_window.width() / 2) - (main_window._available_credits_popup.width() / 2))
            pos.setY((pos.y() + main_window.height() / 2) - (main_window._available_credits_popup.height() / 2))

            main_window._available_credits_popup.move(pos)
            main_window._available_credits_popup.exec_()
            return True
        return False


    def refresh_remote_root_dirs(self, slot):
        api_client = self.get_plugin().get_api_client()
        worker_thread = self.get_worker_thread(u"Fetching Existing Projects...")
        ################################################################################################################

        target = api_client.get_root_directories

        args = set()

        kwargs = {
            'ignore_cache': True
        }

        worker_thread.process(target, args, kwargs, slot)

    def get_icon(self):
        raise NotImplementedError

    def get_plugin_title_image(self):
        raise NotImplementedError

    def get_left_banner_image(self):
        raise NotImplementedError

    def get_left_banner_link(self):
        raise NotImplementedError

    def get_help_url(self):
        raise NotImplementedError

    def use_dark_theme(self):
        return self._use_dark_theme

    def toggle_theme(self):
        self._use_dark_theme = not self._use_dark_theme

        for styled_widget in self._styled_widgets:
            self.set_style_sheet(styled_widget)

    def get_theme(self):
        if self.use_dark_theme():
            return qt_constants.BlackTheme
        else:
            return qt_constants.WhiteTheme

    def set_style_sheet(self, widget, theme=None):

        if widget not in self._styled_widgets:
            self._styled_widgets.append(widget)

        widget.setStyleSheet(self.get_style_sheet(theme=theme))

    def get_style_sheet(self, theme=None):

        if theme is None:
            theme = self.get_theme()

        return u"""
        * {
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QWidget {
            background: transparent;
        }
        
        QWidget:focus {
            outline: 0;
        }
        
        QWidget[main-container-widget=true] {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }
        
        QWidget[background-off-white=true] {
            background: """ + theme.BACKGROUND_COLOR_2 + u""";
        }
        
        QWidget[background-level-0=true] {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }
                
        QWidget[background-level-1=true] {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }
        
        QWidget[background-level-2=true] {
            background: """ + theme.BACKGROUND_COLOR_2 + u""";
        }

        QAbstractScrollArea {
            background: transparent;
        }

        QDialog {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QGroupBox {
            border: 0px solid transparent;
            border-radius: 0px;
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            padding: 15px;
        }

        QGroupBox::title {
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QPushButton {
            font-family: "Montserrat SemiBold";
            background: """ + theme.FOREGROUND_COLOR + u""";
            color: """ + theme.PUSH_BUTTON_COLOR + u""";
            padding: 11px 16px;
            border-radius: 4px;
        }

        QPushButton[large-btn=true] {
            margin-top: 6px;
            min-height: 46px;
        }

        QPushButton:hover {
            color: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QPushButton:pressed {
            color: """ + theme.PUSH_BUTTON_PRESSED_COLOR + u""";
        }

        QPushButton:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton[brand=true] {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton[brand=true]:hover {
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton[brand=true]:pressed {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.GREY_01 + u""";
        }
        
        QPushButton[brand=true]:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton:flat {
            font-family: "Montserrat Medium";
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            padding: 5px 4px;
        }

        QPushButton:flat:hover {
            color: """ + theme.BRAND_COLOR + u""";
        }

        QPushButton:flat:pressed  {
            color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QPushButton:flat:disabled  {
            color: """ + theme.COLOR_DISABLED + u""";
        }
        
        QPushButton::menu-indicator {
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""") 0 0 0 0 stretch stretch;
            subcontrol-origin: padding;
            subcontrol-position: right center;
        }
        
        QPushButton::menu-indicator[noMenuIndicator=true] {
            image: none;
        }
        

        QToolButton::menu-arrow {
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""") 0 0 0 0 stretch stretch;
            subcontrol-origin: padding;
            subcontrol-position: center center;
        }
        
        QToolButton[brand=true] {
            font-family: "Montserrat SemiBold";
            padding: 11px 16px;
            padding-right: 20px; /* make way for the popup button */
            border-radius: 4px;
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.WHITE + u""";
        }

        QToolButton[brand=true]:hover {
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            color: """ + theme.WHITE + u""";
        }

        QToolButton[brand=true]:pressed {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.GREY_01 + u""";
        }
        
        QToolButton[brand=true]:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }
        
        QToolButton::menu-button[brand=true] {
            border: 0px solid transparent;
            width: 20px;
        }
        
        QToolButton::menu-button:hover[brand=true] {
            border: 0px solid transparent;
        }
        

        QFrame {
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            font-family: "Montserrat";
        }

        QFrame[hr=true] {
            min-height: 1px;
            margin: 5px 0px;
            background: """ + theme.BORDER_COLOR + u""";
        }

        QLabel {
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            font-family: "Montserrat";
        }

        QLabel[h1=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 36px;
        }

        QLabel[h2=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 26px;
        }

        QLabel[h3=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 20px;
        }

        QLabel[h4=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 16px;
        }

        QLabel[h5=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 12px;
        }

        QLabel[circle-background-icon=true]{
            background: """ + theme.FOREGROUND_COLOR + u""";
            border-radius: 12px;
            min-width: 24px;
            min-height: 24px;

            max-width: 24px;
            max-height: 24px
        }
        
        QLabel[left-banner=true]{
            padding: 0px;
            background: """ + theme.LEFT_BANNER_BACKGROUND_COLOR + u""";
        }

        QLineEdit {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            margin: 0px 0px 0px 0px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QLineEdit[text=\"\"] { /* appearance of placeholder text */
            font-family: "Montserrat Light";
            font-style: italic;
            color: """ + theme.PLACEHOLDER_TEXT_COLOR + u""";
        }

        QLineEdit:hover {
            border-bottom-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QLineEdit:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QLineEdit:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QSpinBox {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            margin: 0px 0px 0px 0px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QSpinBox:hover {
            border-bottom-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QSpinBox:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QSpinBox::up-button {
            margin: 4px 6px 0px 6px;
            border-image: url(""" + unicode(qt_constants.UP_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QSpinBox::down-button {
            margin: 0px 6px 4px 6px;
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QSpinBox:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QSpinBox::up-button:disabled {
            border-image: url(""" + unicode(qt_constants.UP_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
        }

        QSpinBox::down-button:disabled {
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
        }

        QCheckBox {
            spacing: 5px;
        }

        QCheckBox::indicator {
            width: 18px;
            height: 18px;
            background: transparent;
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
        }

        QCheckBox::indicator:hover {
            border-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QCheckBox::indicator:focus {
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QCheckBox::indicator:unchecked {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
            background: transparent;
        }

        QCheckBox::indicator:unchecked:hover {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QCheckBox::indicator:unchecked:pressed {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QCheckBox::indicator:unchecked:disabled {
            border-color: #ebebeb;
        }

        QCheckBox::indicator:checked {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.CHECK_BOX_CHECKED_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED_BORDER_COLOR + u""";
        }

        QCheckBox::indicator:checked:hover {
            background: """ + theme.CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED__HOVER_BORDER_COLOR + u""";
        }

        QCheckBox::indicator:focus:checked {
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QCheckBox::indicator:checked:disabled {
            background: """ + theme.COLOR_DISABLED + u""";
            border-color: """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QCheckBox::indicator:focus:checked:hover {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QCheckBox::indicator:focus:checked:pressed {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QCheckBox[hidden=true]::indicator {
            border: 0px solid transparent;
            background: transparent;
            image: none;
        }

        QCheckBox[hidden=true]::indicator:checked:disabled {
            border: 0px solid transparent;
            background: transparent;
        }


        QComboBox {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox:hover {
            border-bottom: 1px solid """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QComboBox:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QComboBox:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QComboBox::drop-down {
            subcontrol-origin: padding;
            subcontrol-position: top right;
            width: 15px;

            margin: 3px 0px;
            padding-left: 5px;
            border-left-width: 1px;
            border-left-color: """ + theme.BORDER_COLOR + u""";
            border-left-style: solid; /* just a single line */
        }

        QComboBox::drop-down:hover {
            border-left-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QComboBox::drop-down:focus {
            border-left-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox::drop-down:on {
            border-left-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox::down-arrow {
            margin: 2px 2px 2px 2px;
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
            image: none;
        }

        QComboBox::down-arrow:disabled {
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
            image: none;
        }

        QAbstractItemView {
            color: """ + theme.FOREGROUND_COLOR + u""";
            background: """ + theme.ABSTRACT_ITEM_VIEW_BACKGROUND + u""";
            alternate-background-color: """ + theme.ABSTRACT_ITEM_VIEW_ALTERNATE_BACKGROUND + u""";
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
            border: 1px solid transparent;
            border-radius: 2px;
            outline: 0px;
        }
        
        QAbstractItemView::item{
            height: 28px;
        }
        
        QAbstractItemView::item:focus{
            border: 0px solid transparent;
            outline: 0px;
        }
        
        QAbstractItemView::indicator {
            width: 18px;
            height: 18px;
            background: transparent;
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
        }
        
        QAbstractItemView::indicator:hover {
            border-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QAbstractItemView::indicator:focus {
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView::indicator:unchecked {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
            background: transparent;
        }

        QAbstractItemView::indicator:unchecked:hover {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QAbstractItemView::indicator:unchecked:pressed {
            image: url(""" + unicode(qt_constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QAbstractItemView::indicator:unchecked:disabled {
            border-color: #ebebeb;
        }

        QAbstractItemView::indicator:checked {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.CHECK_BOX_CHECKED_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED_BORDER_COLOR + u""";
        }

        QAbstractItemView::indicator:checked:hover {
            background: """ + theme.CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED__HOVER_BORDER_COLOR + u""";
        }

        QAbstractItemView::indicator:focus:checked {
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView::indicator:checked:disabled {
            background: """ + theme.COLOR_DISABLED + u""";
            border-color: """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QAbstractItemView::indicator:focus:checked:hover {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QAbstractItemView::indicator:focus:checked:pressed {
            image: url(""" + unicode(qt_constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView[hidden=true]::indicator {
            border: 0px solid transparent;
            background: transparent;
            image: none;
        }

        QAbstractItemView[hidden=true]::indicator:checked:disabled {
            border: 0px solid transparent;
            background: transparent;
        }
        
        QHeaderView {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
            border: 0px solid trasparent;
        }
            
        QHeaderView::section {
            background: transparent;
            padding: 10px 8px;
        }
        
        QTreeView::branch {
            padding-right: 4px;
            background: transparent
        }
        
        QTreeView::branch:has-siblings:!adjoins-item {
            background: transparent;
        }
        
        QTreeView::branch:has-siblings:adjoins-item {
            background: transparent;
        }
        
        QTreeView::branch:!has-children:!has-siblings:adjoins-item {
            background: transparent;
        }
        
        QTreeView::branch:closed:has-children:has-siblings {
            image: url(""" + unicode(qt_constants.RIGHT_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:closed:has-children:has-siblings:hover {
            image: url(""" + unicode(qt_constants.RIGHT_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:has-children:!has-siblings:closed {
            image: url(""" + unicode(qt_constants.RIGHT_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:has-children:!has-siblings:closed:hover {
            image: url(""" + unicode(qt_constants.RIGHT_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:open:has-children:has-siblings {
            image: url(""" + unicode(qt_constants.DOWN_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:open:has-children:has-siblings:hover {
            image: url(""" + unicode(qt_constants.DOWN_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:open:has-children:!has-siblings {
            image: url(""" + unicode(qt_constants.DOWN_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }
        
        QTreeView::branch:open:has-children:!has-siblings:hover {
            image: url(""" + unicode(qt_constants.DOWN_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }
        

        QTextEdit:focus{
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            border: 0px solid transparent;
        }

        QMenu {
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
        }

        QMenu::item {
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            padding: 7px 20px 7px 30px;
        }

        QMenu::item:hover {
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QMenu::item:selected {
            color: """ + theme.WHITE + u""";
            background: """ + theme.BRAND_COLOR + u""";
        }


        QMenu::icon { /* appearance of a 'checked' icon */
            padding: 7px 7px;
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
        }

        QMenu::icon:selected {
            background: """ + theme.BRAND_COLOR + u""";
        }

        QMenu::separator {
            height: 1px;
            background: """ + theme.BORDER_COLOR + u""";
            margin-top: 5px;
            margin-bottom: 5px;
            margin-left: 0px;
            margin-right: 0px;
            border: 0px solid transparent;
        }

        /* appearance of the 'checked' icon */
        QMenu::indicator {
            margin-top: 2px;
            margin-bottom: 2px;
            margin-left: 4px;
            min-width: 16px;
            min-height: 16px;
            background: transparent;
        }

        QMenu::indicator:non-exclusive:unchecked {
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:unchecked:selected {
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:checked {
            border: 1px solid """ + theme.BORDER_COLOR_HOVER + u""";
            border-radius: 2px;
            background: """ + theme.BORDER_COLOR_HOVER + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:checked:selected {
            border: 1px solid """ + theme.FOREGROUND_COLOR + u""";
            background: """ + theme.FOREGROUND_COLOR + u""";
            image: none;
        }


        QMenuBar::item {
            padding: 3px 10px;
            background: transparent;
        }

        QMenuBar::item:disabled {
            color: #c4c4c4
        }

        QMenuBar::item:selected { /* when selected using mouse or keyboard */
            color: """ + theme.BRAND_COLOR + u""";
        }

        QMenuBar::item:pressed {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }


        QMenu::item[inPopup]{
            border: 0px;
            min-width: 999px;
        }

        QLabel[user-name-header=true]{
            font-family: "Montserrat Medium";
            font-size: 14px;
        }

        QLabel[invalid=true]{
            color: red;
        }

        QTextEdit[console=true]{
            font-family: "Consolas", "Liberation Mono", "Courier New", "Courier";

            color: """ + theme.WHITE + u""";
            background: """ + theme.CONSOLE_BACKGROUND_COLOR + u""";

            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }
        
        QPlainTextEdit[console=true]{
            font-family: "Consolas", "Liberation Mono", "Courier New", "Courier";

            color: """ + theme.WHITE + u""";
            background: """ + theme.CONSOLE_BACKGROUND_COLOR + u""";

            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QSplitter::handle {
            image: none;
        }


        QScrollBar:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            padding: 15px 0 15px 0;
        }
        
        QScrollBar::handle:vertical {
            background: """ + theme.SCROLL_BAR_HANDLE_BACKGROUND + u""";;
            min-height: 20px;
            opacity: 10;
        }
        
        QScrollBar::add-line:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 15px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
            border-image: url(""" + unicode(qt_constants.DOWN_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::sub-line:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 15px;
            subcontrol-position: top;
            subcontrol-origin: margin;
            border-image: url(""" + unicode(qt_constants.UP_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            width: 3px;
            height: 3px;
            background: none;
        }

        QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none;
        }
        
        
        QScrollBar:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 12px;
            padding: 0 15px 0 15px;
        }
        
        QScrollBar::handle:horizontal {
            background: """ + theme.SCROLL_BAR_HANDLE_BACKGROUND + u""";;
            min-width: 20px;
            opacity: 10;
        }
        
        QScrollBar::add-line:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            subcontrol-position: bottom left;
            subcontrol-origin: margin;
            border-image: url(""" + unicode(qt_constants.LEFT_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::sub-line:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            subcontrol-position: top right;
            subcontrol-origin: margin;
            border-image: url(""" + unicode(qt_constants.RIGHT_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::right-arrow:horizontal, QScrollBar::left-arrow:horizontal {
            width: 3px;
            height: 3px;
            background: none;
        }

        QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
            background: none;
        }
        

        QToolTip{
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 7px;
        }

        QLabel[tool-tip=true]{
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 12px;
        }
        """

    @staticmethod
    def load_fonts():

        for font_file in QtUserInterface.CUSTOM_FONT_FILES:
            font_id = QtGui.QFontDatabase.addApplicationFont(unicode(qt_constants.FONTS_DIRECTORY / font_file))

            if font_id == -1:
                raise ValueError(u"Unable to load font '" + unicode(str(font_file), "utf8") + u"'")

            QtUserInterface.LOADED_FONTS.append(font_id)

    @staticmethod
    def unload_fonts():

        for font_id in QtUserInterface.LOADED_FONTS:
            QtGui.QFontDatabase.removeApplicationFont(font_id)
