__all__ = 'SubmissionViewWidget'

import time

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin.errors import *

from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin.widgets.elided_label import ElidedLabel
from gm_qt_plugin.widgets.qhline import QHLine
from gm_qt_plugin.qt_dialog_preflight_window import PreflightWindow
from gm_qt_plugin.exception_wrapper import ExceptionWrapper
from gm_qt_plugin.qt_none import QtNone


class SubmissionViewWidget(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        def display_job_preset_container():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_job_preset_container()

        # flag to say whether the job preset selector should be displayed
        self._is_job_preset_selector_visible = False

        """ Create Widgets """
        self.title_label = QtWidgets.QLabel("Submission Settings")
        self.title_label.setProperty("h2", True)

        self.description = QtWidgets.QLabel("Submit a job to GridMarkets using the following settings.")
        self.description.setEnabled(False)

        self.qhline = QHLine()

        # hide for v1 of the plugin
        self.title_label.hide()
        self.description.hide()
        self.qhline.hide()

        self.project_label = QtWidgets.QLabel("Project")
        self.project_label.setProperty("h3", True)
        self.project_description = QtWidgets.QLabel("The project to run the selected job against.")
        self.project_description.setEnabled(False)

        self.no_job_presets_icon = QtWidgets.QLabel(u"")
        self.no_job_presets_icon.setPixmap(qt_constants.NEW_JOB_PRESET_ICON_BLACK.pixmap(16))
        self.no_job_presets_icon.setFixedWidth(16)
        # self.no_job_presets_icon.setProperty("circle-background-icon", True)
        self.no_job_presets_icon.setAlignment(QtCore.Qt.AlignCenter)

        self.no_job_presets_label = ElidedLabel("You have not created any job presets to choose from.")
        self.new_job_preset_btn = QtWidgets.QPushButton("Create")
        self.new_job_preset_btn.setFlat(True)
        self.new_job_preset_btn.clicked.connect(display_job_preset_container)
        self.new_job_preset_btn.setMaximumWidth(60)
        self.no_job_preset_widget = QtWidgets.QWidget()

        self.job_label = QtWidgets.QLabel("Job")
        self.job_label.setProperty("h3", True)
        self.job_description = QtWidgets.QLabel("The render settings and output settings to use.")
        self.job_description.setEnabled(False)

        self.group_box = QtWidgets.QGroupBox()
        self.group_box_2 = QtWidgets.QGroupBox()

        self.project_option_selector = QtWidgets.QComboBox()
        self.project_option_selector.currentIndexChanged.connect(self.update_job_options)

        self.job_option_selector = QtWidgets.QComboBox()
        self.job_option_selector.currentIndexChanged.connect(self.redraw_job_preset)
        self._view_sleected_job_preset_btn = QtWidgets.QPushButton(u"")
        self._view_sleected_job_preset_btn.setIcon(qt_constants.EYE_ICON_BLACK)
        self._view_sleected_job_preset_btn.setFlat(True)
        self._view_sleected_job_preset_btn.clicked.connect(self.view_job_preset_slot)
        self._view_sleected_job_preset_btn.setFixedHeight(21)
        self._view_sleected_job_preset_btn.setFixedWidth(23)

        self.job_preset_selection_widget = QtWidgets.QWidget()
        self.job_preset_selection_widget.hide()

        # create a project instance builder
        cached_api_schema = plugin.get_api_client().get_cached_api_schema()
        project_definition_schema = cached_api_schema.get_project_definition_schema()
        project_instance_builder_factory = plugin.get_factory_collection().get_project_instance_builder_factory()
        self.project_instance_builder = project_instance_builder_factory.get_project_instance_builder(
            project_definition_schema)

        self.reload_scene_settings_btn = QtWidgets.QPushButton(u"Reload Scene Settings")
        self.reload_scene_settings_btn.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.reload_scene_settings_btn.clicked.connect(self.reload_scene_settings)
        self.reload_movie = QtGui.QMovie(unicode(qt_constants.SPINNER_WHITE_666666_GIF_PATH))

        self.submit_btn = QtWidgets.QPushButton(u"Submit")
        self.submit_btn.setProperty(qt_constants.STYLE_PROPS.BRAND, True)
        self.submit_btn.setMinimumWidth(300)
        self.submit_btn.clicked.connect(self._submit)

        user_interface = plugin.get_user_interface()
        user_interface.operation_started.connect(lambda: self.submit_btn.setDisabled(True))
        user_interface.operation_finished.connect(lambda: self.submit_btn.setEnabled(True))

        """ Create Layouts """
        self.v1_widget = QtWidgets.QWidget()
        self.v1_layout = QtWidgets.QVBoxLayout()
        self.v1_layout.setContentsMargins(0, 0, 0, 0)
        self.v1_layout.addWidget(self.project_label)
        self.v1_layout.addWidget(self.project_description)
        self.v1_layout.addWidget(self.project_option_selector)
        self.v1_widget.setLayout(self.v1_layout)

        self.job_preset_warning_layout = QtWidgets.QHBoxLayout()
        self.job_preset_warning_layout.setContentsMargins(0, 0, 0, 0)
        self.job_preset_warning_layout.addWidget(self.no_job_presets_icon)
        self.job_preset_warning_layout.addWidget(self.no_job_presets_label)
        self.job_preset_warning_layout.addWidget(self.new_job_preset_btn)
        self.no_job_preset_widget.setLayout(self.job_preset_warning_layout)

        self.job_preset_selector_layout = QtWidgets.QHBoxLayout()
        self.job_preset_selector_layout.setContentsMargins(0, 0, 0, 0)
        self.job_preset_selector_layout.addWidget(self.job_option_selector)
        self.job_preset_selector_layout.addWidget(self._view_sleected_job_preset_btn)
        self.job_preset_selection_widget.setLayout(self.job_preset_selector_layout)

        self.v2_widget = QtWidgets.QWidget()
        self.v2_layout = QtWidgets.QVBoxLayout()
        self.v2_layout.setContentsMargins(0, 0, 0, 0)
        self.v2_layout.addWidget(self.job_label)
        self.v2_layout.addWidget(self.job_description)
        self.v2_layout.addWidget(self.job_preset_selection_widget)
        self.v2_layout.addWidget(self.no_job_preset_widget)
        self.v2_widget.setLayout(self.v2_layout)

        self.h_layout = QtWidgets.QHBoxLayout()
        self.h_layout.setSpacing(qt_constants.MARGIN_SIZE * 2)
        self.h_layout.addWidget(self.v1_widget)
        self.h_layout.addWidget(self.v2_widget)
        self.h_layout.setStretchFactor(self.v1_widget, 1)
        self.h_layout.setStretchFactor(self.v2_widget, 1)

        self.container_layout = QtWidgets.QVBoxLayout(self)
        self.container_layout.addLayout(self.h_layout)

        self.submit_btn_layout = QtWidgets.QHBoxLayout(self)
        self.submit_btn_layout.setAlignment(QtCore.Qt.AlignRight)
        self.submit_btn_layout.addWidget(self.reload_scene_settings_btn)
        self.submit_btn_layout.addWidget(self.submit_btn)

        self.group_box.setLayout(self.container_layout)
        self.group_box_2.setLayout(self.submit_btn_layout)

        # temporary layout for showing the job preset settings in
        self.job_preset_layout = QtWidgets.QVBoxLayout()

        self.layout = QtWidgets.QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.title_label)
        self.layout.addWidget(self.description)
        self.layout.addWidget(self.qhline)

        # hide project / job preset selection for v1
        # self.layout.addWidget(self.group_box)

        self.layout.addLayout(self.job_preset_layout)
        self.layout.addWidget(self.group_box_2)

        self.update_project_options()

    def reload_scene_settings(self):

        def set_icon(frame):
            pixmap = self.reload_movie.currentPixmap()

            icon = QtGui.QIcon()
            icon.addPixmap(pixmap, QtGui.QIcon.Normal)
            icon.addPixmap(pixmap, QtGui.QIcon.Disabled)

            self.reload_scene_settings_btn.setIcon(icon)

        # disable buttons
        self.submit_btn.setDisabled(True)
        self.reload_scene_settings_btn.setDisabled(True)

        # start reload spinner
        self.reload_movie.start()
        self.reload_movie.frameChanged[int].connect(set_icon)

        job_preset = self.job_option_selector.currentData()

        worker_thread = self.get_plugin().get_user_interface().get_worker_thread(u"Reloading scene settings...")

        ################################################################################################################

        def target(job_preset):
            time_start = time.time()

            # Attempt to infer the value of all application and user defined job preset attributes.
            # This is so any api requests made by the application_attribute_source can be cached before redrawing the
            # job preset
            for job_preset_attribute in job_preset.get_job_preset_attributes():

                inference_source = job_preset_attribute.get_inference_source()
                is_app_defined = inference_source == gm_meta_plugin.InferenceSource.get_application_inference_source()
                is_user_defined = inference_source == gm_meta_plugin.InferenceSource.get_user_defined_inference_source()

                if is_app_defined or is_user_defined:
                    key = job_preset_attribute.get_key()

                    # enum items
                    try:
                        self.get_plugin().get_application_attribute_source().get_enum_items(None, None, key)
                    except gm_meta_plugin.errors.ApplicationAttributeNotFound as e:
                        pass

                    # attribute values
                    try:
                        self.get_plugin().get_application_attribute_source().get_attribute_value(None, None, key)
                    except gm_meta_plugin.errors.ApplicationAttributeNotFound as e:
                        pass

            time_end = time.time()

            # get elapsed time in seconds
            total_time = time_end - time_start

            # if elapsed time is less than RELOAD_SCENE_SETTINGS_TIMER_INTERVAL then sleep for the difference. This is
            # to give the user some visual feedback that something has happened
            if total_time < qt_constants.RELOAD_SCENE_SETTINGS_SLEEP_INTERVAL:
                time.sleep(qt_constants.RELOAD_SCENE_SETTINGS_SLEEP_INTERVAL - total_time)

            return True

        args = (
            job_preset,
        )

        kwargs = {}

        worker_thread.process(target, args, kwargs, self._reload_scene_settings_handler)

    def _reload_scene_settings_handler(self, result):
        # enable buttons
        self.submit_btn.setEnabled(True)
        self.reload_scene_settings_btn.setEnabled(True)

        # stop reload spinner
        self.reload_movie.frameChanged[int].disconnect()
        self.reload_scene_settings_btn.setIcon(QtGui.QIcon())

        if isinstance(result, bool) and result == True:
            self.redraw_job_preset()
            return True

        return False

    def update_project_options(self):
        # get all possible remote project options
        remote_project_container = self.get_plugin().get_remote_project_container()

        remote_projects = remote_project_container.get_all()

        current_selected_remote_project = self.project_option_selector.currentData()
        if current_selected_remote_project is None:
            current_selected_remote_project = qt_constants.CURRENT_SCENE_PROJECT_OPTION_INDEX

        self.project_option_selector.clear()
        self.project_option_selector.addItem(qt_constants.MAYA_ICON,
                                             "Submit Scene as new Project",
                                             qt_constants.CURRENT_SCENE_PROJECT_OPTION_INDEX)

        for i, remote_project in enumerate(remote_projects):
            icon = qt_utils.get_product_icon(remote_project.get_attribute(api_constants.API_KEYS.APP))

            self.project_option_selector.addItem(icon,
                                                 remote_project.get_name(),
                                                 remote_project)

            if remote_project is current_selected_remote_project:
                self.project_option_selector.setCurrentIndex(i + 1)

        if not remote_projects:
            self.project_option_selector.setCurrentIndex(0)

        self.update_job_options()

    def get_selected_project_option(self):
        remote_project = self.project_option_selector.currentData()

        if remote_project is None:
            remote_project = qt_constants.CURRENT_SCENE_PROJECT_OPTION_INDEX

        return remote_project

    def update_job_options(self):
        # get all possible job preset options
        job_preset_container = self.get_plugin().get_preferences_container().get_job_preset_container()
        job_presets = job_preset_container.get_all()

        # get selected project option
        remote_project = self.get_selected_project_option()

        # get project type from project
        if remote_project == qt_constants.CURRENT_SCENE_PROJECT_OPTION_INDEX:
            # get project type id of the current scene
            project_type_id = self.get_plugin().get_packed_scene_builder().get_project_type_id()
        else:
            # get selected projects project type id
            project_type_id = remote_project.get_attribute(api_constants.API_KEYS.PROJECT_TYPE_ID)

        api_schema = self.get_plugin().get_api_client().get_cached_api_schema()
        project_attribute = api_schema.get_project_definition_schema().get_project_attribute_with_id(project_type_id)
        compatible_job_presets = []

        for job_preset in job_presets:
            if job_preset.get_job_definition() in project_attribute.get_compatible_job_definitions():
                compatible_job_presets.append(job_preset)

        if compatible_job_presets:

            # Add the compatible job presets to the job_preset_selector while maintaining the selected job preset if it
            # is still in the compatible list
            current_selected_job_preset = self.job_option_selector.currentData()
            self.job_option_selector.clear()
            for i, job_preset in enumerate(compatible_job_presets):

                self.job_option_selector.addItem(job_preset.get_icon(), job_preset.get_name(), job_preset)

                if job_preset is current_selected_job_preset:
                    self.job_option_selector.setCurrentIndex(i)

            self.show_job_preset_selector()
        else:
            self.hide_job_preset_selector()

        self.redraw_job_preset()

    def show_job_preset_selector(self):
        self.no_job_preset_widget.hide()
        self.job_preset_selection_widget.show()
        self._is_job_preset_selector_visible = True

    def hide_job_preset_selector(self):
        self.no_job_preset_widget.show()
        self.job_preset_selection_widget.hide()
        self._is_job_preset_selector_visible = False

    def is_job_preset_selector_visible(self):
        return self._is_job_preset_selector_visible

    def redraw_job_preset(self):
        # get the focused job preset
        job_preset = self.job_option_selector.currentData()

        # build the new layout
        if job_preset and self.is_job_preset_selector_visible():
            self.job_preset_layout.addWidget(job_preset)
            job_preset.update_app_inferred_values()

    def _submit(self):
        user_interface = self.get_plugin().get_user_interface()

        # Prevent submit if already running an operation in another thread
        if user_interface.is_running_operation():
            QtWidgets.QMessageBox.warning(self,
                                          u"Invalid Project Name",
                                          u"The plugin is already busy running another operation, wait for the first "
                                          "operation to finish and then try again.",
                                          QtWidgets.QMessageBox.Ok,
                                          defaultButton=QtWidgets.QMessageBox.Ok)
            return

        remote_project = self.get_selected_project_option()
        job_preset = self.job_option_selector.currentData()

        # if no job preset is selected ask the user if they want to create one
        if job_preset is None:
            reply = QtWidgets.QMessageBox.information(self,
                                                      u"Create Job Preset?",
                                                      u"No job preset selected.\nWould you like to create a new job preset?",
                                                      QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                      defaultButton=QtWidgets.QMessageBox.Yes)

            if reply == QtWidgets.QMessageBox.Yes:
                self.get_plugin().get_user_interface().get_main_switcher_ui().display_job_preset_container()
                return
            else:
                return

        # validate the job preset
        is_valid, validation_message = self.get_plugin().get_job_preset_validator().validate(job_preset)
        if not is_valid:
            QtWidgets.QMessageBox.warning(self,
                                          u"Invalid Job Preset",
                                          validation_message,
                                          QtWidgets.QMessageBox.Ok,
                                          defaultButton=QtWidgets.QMessageBox.Ok)
            return

        if remote_project == qt_constants.CURRENT_SCENE_PROJECT_OPTION_INDEX:
            self._submit_to_new_project(job_preset)
        else:
            self._submit_to_existing_project(remote_project, job_preset)

    def _execute_submit(self, action, packed_project, job_preset):

        api_client = self.get_plugin().get_api_client()

        if action == PreflightWindow.UPLOAD_AND_SUBMIT:
            return api_client.submit_new_project(packed_project, job_preset, delete_local_files_after_upload=False)

        if action == PreflightWindow.UPLOAD:
            return api_client.upload_project(packed_project, False, delete_local_files_after_upload=False)

        if action == PreflightWindow.SUBMIT:
            remote_project = gm_meta_plugin.gridmarkets.remote_project.convert_packed_project(packed_project,
                                                                                              self.get_plugin())
            return api_client.submit_to_remote_project(remote_project, job_preset)

    def _execute_pack_new_project(self,
                                  project_name,
                                  attributes,
                                  job_preset):

        packed_scene_builder = self.get_plugin().get_packed_scene_builder()

        product = attributes[api_constants.API_KEYS.APP]
        product_version = attributes[api_constants.API_KEYS.APP_VERSION]
        packed_project = packed_scene_builder.get_current_scene(project_name,
                                                                product,
                                                                product_version,
                                                                attributes,
                                                                job_preset=job_preset)

        return packed_project, job_preset

    def _submit_to_new_project(self, job_preset):
        plugin = self.get_plugin()
        user_interface = plugin.get_user_interface()
        api_client = plugin.get_api_client()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        # get the project name
        try:
            # try to get the project name from the job preset itself
            project_name_attribute = job_preset.get_job_preset_attribute_by_key(api_constants.API_KEYS.PROJECT_NAME)
            project_name = project_name_attribute.get_value()

            # Validate the project name input
            if not project_name:
                QtWidgets.QMessageBox.warning(self,
                                              u"Invalid Project Name",
                                              u"No project name provided",
                                              QtWidgets.QMessageBox.Ok,
                                              defaultButton=QtWidgets.QMessageBox.Ok)
                return

        except ValueError:
            # if the job preset does not have a project name attribute then prompt the user for the name
            project_name, ok = QtWidgets.QInputDialog.getText(self,
                                                              u"Project Name",
                                                              u"Enter the name of the new Project:",
                                                              mode=QtWidgets.QLineEdit.Normal,
                                                              text="Project 0")

            if not ok:
                return

        # if project already exists warn the user
        root_directories = api_client.get_root_directories(ignore_cache=True)
        if project_name in root_directories:
            msb_box = QtWidgets.QMessageBox.warning(self,
                                                    u"Project Already Exists",
                                                    u"A project already exists with that name.\nWould you like to "
                                                    "submit anyway? Any project files with the same name will be "
                                                    "overwritten.",
                                                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                    defaultButton=QtWidgets.QMessageBox.Yes)

            if msb_box == QtWidgets.QMessageBox.Yes:
                pass
            else:
                return

        self.project_instance_builder.update_states(project_name=project_name)
        attributes = self.project_instance_builder.get_project_attributes(force_transition=True)

        logger.info(u"Packing new project...")

        worker_thread = user_interface.get_worker_thread(u"Packing Scene...")

        ################################################################################################################

        target = self._execute_pack_new_project

        args = (
            project_name,
            attributes,
            job_preset
        )

        kwargs = {}

        worker_thread.process(target, args, kwargs, self._handle_packed_project_result)

    def _submit_to_existing_project(self, remote_project, job_preset):
        plugin = self.get_plugin()
        user_interface = plugin.get_user_interface()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        logger.info(u"Submitting Job...")

        worker_thread = user_interface.get_worker_thread(u"Submitting job...")

        ################################################################################################################

        target = plugin.get_api_client().submit_to_remote_project

        args = (
            remote_project,
            job_preset
        )

        kwargs = {}

        worker_thread.process(target, args, kwargs, self._handle_result_slot)

    def _handle_packed_project_result(self, result):
        plugin = self.get_plugin()
        user_interface = plugin.get_user_interface()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        if isinstance(result, tuple) and \
                len(result) == 2 and \
                isinstance(result[0], gm_meta_plugin.PackedProject) and \
                isinstance(result[1], gm_meta_plugin.JobPreset):

            packed_project = result[0]
            job_preset = result[1]

            # hide the main window while the preflight window is open
            user_interface.get_main_window().hide()

            # open preflight window
            preflight_window = PreflightWindow(self.get_plugin(), packed_project, parent=self)
            preflight_window.exec_()

            # show the main window once the preflight window has closed
            user_interface.get_main_window().show()

            # if the user has pressed cancel or closed the window then stop the submission
            if preflight_window.cancel_submission():
                QtWidgets.QMessageBox.information(self,
                                                  u"",
                                                  u"Submission canceled.",
                                                  QtWidgets.QMessageBox.Ok,
                                                  defaultButton=QtWidgets.QMessageBox.Ok)
                return True

            # otherwise update the packed projects files to only contain the files selected by the user for upload
            files_to_upload = set(preflight_window.get_files_to_upload())
            next_action = preflight_window.get_next_action()

            if next_action == PreflightWindow.UPLOAD_AND_SUBMIT or next_action == PreflightWindow.UPLOAD:
                # check the user has selected at least one file to upload
                if not files_to_upload:
                    QtWidgets.QMessageBox.warning(self,
                                                  u"Warning",
                                                  u"No Files selected for upload, please select at least one file before uploading.",
                                                  QtWidgets.QMessageBox.Ok,
                                                  defaultButton=QtWidgets.QMessageBox.Ok)
                    return True

            packed_project.set_files(files_to_upload)

            if next_action == PreflightWindow.UPLOAD_AND_SUBMIT:
                logger.info(u"Uploading new project and submitting job...")
                result_handler = self._handle_submit_and_upload_slot

            elif next_action == PreflightWindow.UPLOAD:
                logger.info(u"Uploading new project...")
                result_handler = self._handle_upload_project_slot

            elif next_action == PreflightWindow.SUBMIT:
                logger.info(u"Submitting job...")
                result_handler = self._handle_submit_slot

            else:
                raise ValueError(u"Unknown preflight action '" + unicode(next_action) + u"'")

            worker_thread = user_interface.get_worker_thread(u"Submitting...")

            ################################################################################################################

            target = self._execute_submit

            args = (
                next_action,
                packed_project,
                job_preset
            )

            kwargs = {}

            worker_thread.process(target, args, kwargs, result_handler)
            return True

        if isinstance(result, ExceptionWrapper):
            exception = result.get_exception()

            if isinstance(exception, FilePackingError):
                self._log_error(logger, exception.user_message)
                return True

        return False

    def _log_error(self, logger, message):
        logger.exception(message)
        QtWidgets.QMessageBox.warning(self,
                                      u"Warning",
                                      message,
                                      QtWidgets.QMessageBox.Ok,
                                      defaultButton=QtWidgets.QMessageBox.Ok)

    def _handle_upload_project_slot(self, result):
        plugin = self.get_plugin()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        if isinstance(result, ExceptionWrapper):
            exception = result.get_exception()

            if isinstance(exception, NotSignedInError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, NotSignedInError):
                msg = u"Could not upload your current scene. You are not signed in."
                self._log_error(logger, msg)
                return True

            if isinstance(exception, APIClientError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, UnsupportedApplicationError):
                self._log_error(logger, exception.user_message)
                return True

        # a remote project is returned when submitting a new project or uploading a project
        if isinstance(result, gm_meta_plugin.RemoteProject):
            QtWidgets.QMessageBox.information(self,
                                              u"",
                                              u"Project upload started.\nCheck Envoy to see your project's upload status.",
                                              QtWidgets.QMessageBox.Ok,
                                              defaultButton=QtWidgets.QMessageBox.Ok)
            return True

        return False

    def _handle_submit_slot(self, result):
        plugin = self.get_plugin()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        if isinstance(result, ExceptionWrapper):
            exception = result.get_exception()

            if isinstance(exception, NotSignedInError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, NotSignedInError):
                msg = u"Could not upload your current scene. You are not signed in."
                self._log_error(logger, msg)
                return True

            if isinstance(exception, APIClientError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, UnsupportedApplicationError):
                self._log_error(logger, exception.user_message)
                return True

        if isinstance(result, QtNone):  # None is returned when submitting to an existing project
            QtWidgets.QMessageBox.information(self,
                                              u"",
                                              u"Your job has been submitted.",
                                              QtWidgets.QMessageBox.Ok,
                                              defaultButton=QtWidgets.QMessageBox.Ok)
            return True

        return False

    def _handle_submit_and_upload_slot(self, result):
        plugin = self.get_plugin()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        if isinstance(result, ExceptionWrapper):
            exception = result.get_exception()

            if isinstance(exception, NotSignedInError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, NotSignedInError):
                msg = u"Could not upload your current scene. You are not signed in."
                self._log_error(logger, msg)
                return True

            if isinstance(exception, APIClientError):
                self._log_error(logger, exception.user_message)
                return True

            if isinstance(exception, UnsupportedApplicationError):
                self._log_error(logger, exception.user_message)
                return True

        # a remote project is returned when submitting a new project or uploading a project
        if isinstance(result, gm_meta_plugin.RemoteProject):
            QtWidgets.QMessageBox.information(self,
                                              u"",
                                              u"Project upload started.\nYour job will start when your files have "
                                              "finished uploading.\nCheck Envoy to see your project's upload status.",
                                              QtWidgets.QMessageBox.Ok,
                                              defaultButton=QtWidgets.QMessageBox.Ok)
            return True

        return False

    def view_job_preset_slot(self):
        job_preset = self.job_option_selector.currentData()

        if job_preset is not None:
            job_preset_container = self.get_plugin().get_preferences_container().get_job_preset_container()
            job_preset_container.focus_item(job_preset)

            self.get_plugin().get_user_interface().get_main_switcher_ui().display_job_preset_container()
