__all__ = 'UploadCurrentSceneWidget'

from PySide2 import QtCore
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin import qt_utils
from gm_meta_plugin.errors import *


class UploadCurrentSceneWidget(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        """ Create Widgets """
        self.group_box = QtWidgets.QGroupBox()
        self.group_box.setTitle("Upload Current Scene")

        self.description_label = QtWidgets.QLabel("Upload your current scene as a new project to GridMarkets.")
        self.description_label.setEnabled(False)

        cached_api_schema = plugin.get_api_client().get_cached_api_schema()
        self.project_attribute_widgets = cached_api_schema.create_project_attribute_widgets()

        self.project_name_label = QtWidgets.QLabel(u"Project Name")
        self.project_name_widget = self.project_attribute_widgets[api_constants.PROJECT_ATTRIBUTE_IDS.PROJECT_NAME]

        self.product_version_label = QtWidgets.QLabel(u"Version")
        self.product_version_widget = self.project_attribute_widgets[api_constants.PROJECT_ATTRIBUTE_IDS.MAYA_VERSION]

        self.upload_btn = QtWidgets.QPushButton("Upload Current Scene")
        self.upload_btn.clicked.connect(self.execute)

        """ Create Layouts """
        self.form_layout = QtWidgets.QFormLayout()
        self.form_layout.addRow(self.project_name_label, self.project_name_widget)
        self.form_layout.addRow(self.product_version_label, self.product_version_widget)

        self.container_layout = QtWidgets.QVBoxLayout()
        self.container_layout.addWidget(self.description_label)
        self.container_layout.addLayout(self.form_layout)
        self.container_layout.addWidget(self.upload_btn)

        self.group_box.setLayout(self.container_layout)

        self.layout = QtWidgets.QVBoxLayout(self)
        self.layout.setContentsMargins(0,0,0,0)
        self.layout.setAlignment(QtCore.Qt.AlignTop)

        self.layout.addWidget(self.group_box)

    def execute(self):
        plugin = self.get_plugin()
        api_client = plugin.get_api_client()
        user_interface = plugin.get_user_interface()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        widgets = self.project_attribute_widgets

        packed_project_builder = self.get_plugin().get_packed_scene_builder()

        # get project name
        project_name = self.project_name_widget.get_value()

        # if project already exists warn the user
        root_directories = api_client.get_root_directories(ignore_cache=True)
        if project_name in root_directories:
            msb_box = QtWidgets.QMessageBox.warning(self,
                                                    u"Project Already Exists",
                                                    u"A project already exists with that name.\nWould you like to "
                                                    "submit anyway? Any project files with the same name will be "
                                                    "overwritten.",
                                                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                    defaultButton=QtWidgets.QMessageBox.Yes)

            if msb_box == QtWidgets.QMessageBox.Yes:
                pass
            else:
                return

        # set and get product
        product_widget = widgets[api_constants.PROJECT_ATTRIBUTE_IDS.PRODUCT]
        product_widget.set_value(packed_project_builder.get_product_type())
        product = product_widget.get_value()

        # get product version
        product_version = self.product_version_widget.get_value()

        # set the other attributes to their application inferred value
        app_attribute_source = plugin.get_application_attribute_source()
        app_attribute_source.set_project_attribute_values(project_name, product, product_version, widgets=widgets)

        attributes = qt_utils.get_project_attributes(plugin,
                                                     widgets=widgets,
                                                     force_transition=True)

        logger.info(u"Uploading Current Scene...")

        worker_thread = user_interface.get_worker_thread(u"Uploading project...")

        ################################################################################################################

        target = self._execute

        args = (
            project_name,
            product,
            product_version,
            attributes
        )

        kwargs = {}

        worker_thread.process(target, args, kwargs, self._handle_result_slot)

    def _execute(self, project_name, product, product_version, attributes):
        packed_scene_builder = self.get_plugin().get_packed_scene_builder()
        packed_project = packed_scene_builder.get_current_scene(project_name, product, product_version, attributes)

        api_client = self.get_plugin().get_api_client()
        return api_client.upload_project(packed_project, True, delete_local_files_after_upload=False)

    def _handle_result_slot(self, result):
        logger = self.get_plugin().get_logging_coordinator().get_logger(__name__)

        if isinstance(result, gm_meta_plugin.RemoteProject):
            # add remote project to container
            plugin = self.get_plugin()
            plugin.get_remote_project_container().append(result)

            # change view to remote projects view
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_remote_project_container()
            return True

        elif type(result) == FilePackingError:
            msg = u"Could not finish packing your project. " + result.user_message
            logger.error(msg)
            return True

        elif type(result) == NotSignedInError:
            msg = u"Could not upload your current scene. You are not signed in."
            logger.error(msg)
            return True

        elif type(result) == APIClientError:
            msg = u"Could not upload your current scene. " + result.user_message
            logger.error(msg)
            return True

        return False
