import os
import sys
import imp
from imp import PKG_DIRECTORY

import maya.api.OpenMaya as OpenMaya

# map of maya plugin modules
MAYA_PLUGIN_PACKAGE_NAME = "gm_maya_plugin"
MAYA_PLUGIN_MODULE_NAMES = ['__init__']
MAYA_PLUGIN_MODULE_FULL_NAMES = map(lambda name: MAYA_PLUGIN_PACKAGE_NAME + '.' + name, MAYA_PLUGIN_MODULE_NAMES)

# map of all modules
ALL_MODULE_NAMES = MAYA_PLUGIN_MODULE_NAMES
ALL_MODULE_FULL_NAMES = MAYA_PLUGIN_MODULE_FULL_NAMES

# tuple of modules to reload
MODULES_TO_RELOAD = ("gm_meta_plugin", "gm_qt_plugin", MAYA_PLUGIN_PACKAGE_NAME)


def find_module(module, paths=None):
    """Just like 'imp.find_module()', but with package support"""

    parts = module.split('.')

    while parts:
        part = parts.pop(0)
        f, path, (suffix, mode, kind) = info = imp.find_module(part, paths)

        if kind == PKG_DIRECTORY:
            parts = parts or ['__init__']
            paths = [path]

        elif parts:
            raise ImportError("Can't find %r in %s" % (parts, module))

    return info


def load_modules(module_names, module_full_names):
    for module_name, module_full_name in zip(module_names, module_full_names):
        if module_full_name in sys.modules:
            # reload already imported modules to make sure they update after code changes (useful for development)
            reload(sys.modules[module_full_name]) in globals()

        else:
            # import module if not already imported
            info = find_module(module_full_name)

            module = imp.load_module(module_name, info[0], info[1], info[2])

            globals()[module_full_name] = module
            setattr(globals()[module_full_name], 'modulesNames', module_full_names)


##########################################################
# Plug-in initialization.
##########################################################

gm_plugin_main_window = None


def maya_useNewAPI():
    """
    The presence of this function tells Maya that the plugin produces, and
    expects to be passed, objects created using the Maya Python API 2.0.
    """
    pass


def create_and_open_plugin():
    from gm_maya_plugin import PluginFetcher
    from gm_maya_plugin import maya_utils
    from gm_qt_plugin.qt_dialog_main_window import MainWindow

    # close any existing main plugin window
    global gm_plugin_main_window
    if gm_plugin_main_window:
        gm_plugin_main_window.close()

    # delete the underlying plugin if it already exists
    PluginFetcher.delete_cached_plugin()

    # create the plugin
    plugin = PluginFetcher.get_plugin()

    # get the main maya window
    maya_main_widow = maya_utils.maya_main_window()

    # create the main plugin window
    main_window = MainWindow(plugin, create_and_open_plugin, parent=maya_main_widow)

    # if the maya plugin menu item already exists
    if maya_utils.does_plugin_menu_item_exist():
        # update its actions so that they are pointing to the new main window
        maya_utils.update_maya_menu_bar_actions(main_window)
    else:
        # otherwise create the menu bar item
        maya_utils.create_maya_menu_bar_btn(main_window)

    # store the new window in a global variable
    gm_plugin_main_window = main_window

    # return the main window
    return gm_plugin_main_window


def initializePlugin(mobject):
    ''' Initialize the plug-in when Maya loads it. '''

    # try to uninitialise the plug-in if it already exists
    try:
        uninitializePlugin(mobject)
    except ImportError as e:
        # An import error will be raised if the plug-in has not already been initialised
        pass

    mplugin = OpenMaya.MFnPlugin(mobject, vendor='GridMarkets Pte. Ltd.', version='1.3.1')

    MODULE_PLUGINS_PATH = os.path.abspath(mplugin.loadPath())
    MODULE_LIBRARIES_PATH = os.path.join(os.path.dirname(MODULE_PLUGINS_PATH), "lib")
    MODULE_SOURCES_PATH = os.path.join(os.path.dirname(MODULE_PLUGINS_PATH), "sources")

    if MODULE_SOURCES_PATH not in sys.path:
        sys.path.insert(0, MODULE_SOURCES_PATH)

    if MODULE_LIBRARIES_PATH not in sys.path:
        sys.path.insert(0, MODULE_LIBRARIES_PATH)

    load_modules(ALL_MODULE_NAMES, ALL_MODULE_FULL_NAMES)

    # create and open the main plugin window
    create_and_open_plugin()

def uninitializePlugin(mobject):
    ''' Uninitialize the plug-in when Maya un-loads it. '''
    from gm_maya_plugin import maya_utils

    mplugin = OpenMaya.MFnPlugin(mobject)

    maya_utils.remove_maya_menu_bar_btn()

    # close the existing gm plugin window assuming if it exists
    global gm_plugin_main_window
    if gm_plugin_main_window is not None:
        gm_plugin_main_window.close_window()
        gm_plugin_main_window.deleteLater()
        del gm_plugin_main_window

    keys_to_delete = []
    for key, value in sys.modules.iteritems():
        if key.startswith(MODULES_TO_RELOAD):
            keys_to_delete.append(key)

    for key in keys_to_delete:
        del(sys.modules[key])
