__all__ = 'PackedSceneBuilder'

import datetime
import sys
import pathlib
import gm_meta_plugin

import maya
import maya.cmds as cmds

from gm_meta_plugin.errors import FilePackingError
from gm_meta_plugin.gridmarkets import constants as api_constants

from gm_maya_plugin import maya_constants, maya_utils


class PackedSceneBuilder(gm_meta_plugin.PackedSceneBuilder):

    def __init__(self, plugin):
        self._plugin = plugin

    def get_plugin(self):
        return self._plugin

    def _build_scene_internal(self, project_name, product, product_version, attributes, job_preset=None):
        fileName = cmds.file(query=True, sceneName=True)

        # If the scene has not been saved
        if fileName == "":
            raise FilePackingError(u"Scene file not saved.")

        # If the scene has been created, saved and then erased from the disk
        if cmds.file(query=True, exists=True) == 0:
            raise FilePackingError(u"Scene file does not exist on filesystem.")

        # If the scene has been modified then save the scene
        if cmds.file(query=True, anyModified=True):
            maya.utils.executeInMainThreadWithResult("maya.cmds.file(save=True)")

        # get the project root dir
        root_dir = maya_utils.get_project_dir()

        # get a list of all the files associated with the scene
        project_files = maya.utils.executeInMainThreadWithResult(
            "maya.cmds.file(query=True, list=True, withoutCopyNumber=True)")

        # Find all referenced files of the current scene. For any unloaded reference, load them first, get file list
        # that should be archived and then restore its unloaded status. Use ls() instead of file(q=True, r=True) because
        # it also gets nested references.
        reference_nodes = maya.utils.executeInMainThreadWithResult("maya.cmds.ls(type='reference')")
        for ref_node in reference_nodes:
            if ref_node.find('sharedReferenceNode') == -1:
                try:
                    is_loaded = maya.utils.executeInMainThreadWithResult(
                        lambda: cmds.referenceQuery(ref_node, isLoaded=True))
                except Exception:
                    # Maya has a bug where some file references exist but don't reference anything. The throw an
                    # exception if you try to query them along the lines of.
                    # "Reference node '<name>' is not associated with a reference file."
                    continue

                # Skip unloaded references
                if not is_loaded:
                    continue

                # Get all external files related to this reference
                files_of_this_ref = maya.utils.executeInMainThreadWithResult(
                    "maya.cmds.file(query=True, list=True, withoutCopyNumber=True)")
                for file_of_this_ref in files_of_this_ref:
                    project_files.append(file_of_this_ref)

        # add the project workspace.mel file
        workspacePath = maya.utils.executeInMainThreadWithResult(
            "maya.cmds.workspace(query=True, fullName=True) + '/workspace.mel'")
        project_files.append(workspacePath)

        # IO boundary check. Encode file paths based on system file encoding
        project_files = list(map(lambda x: x.encode(sys.getfilesystemencoding()), project_files))

        # remove empty file references (e.g. "")
        project_files = list(filter(lambda x: bool(x), project_files))

        # convert to a set of pathlib.Path objects
        project_files = set(map(lambda f: pathlib.Path(f), project_files))

        # if a file is not under the root directory then raise an error
        external_files = []
        for project_file in list(project_files):
            if root_dir not in project_file.parents:
                external_files.append(project_file)

        if external_files:
            msg = u"Warning - Your scene contains external files.\n" \
                  u"Please move the following files into your project directory:\n"

            counter = 0
            while external_files and counter < 5:
                external_file = external_files.pop()
                counter = counter + 1

                msg = msg + u"   " + unicode(str(external_file), sys.getfilesystemencoding()) + "\n"

            if external_files:
                msg = msg + u"   ({0} files not shown)\n".format(len(external_files))

            root_dir_unicode = unicode(str(root_dir), sys.getfilesystemencoding())
            msg = msg + u"\nYour project directory is:\n   {0}\n\n" \
                        u"Either move the file into your Maya project folder or set your Maya project to a different " \
                        u"location using 'File -> Set Project...'".format(root_dir_unicode)

            raise FilePackingError(msg)

        # set the path attribute
        attributes[api_constants.API_KEYS.PATH] = fileName

        # Set the remote output folder path.
        # Note that the path must be of the format /<project_name>/render_results/<folder_name> for Maya
        timestamp = datetime.datetime.now()
        submission_name_timestamp = unicode(timestamp.strftime("%Y-%b-%d %H:%M:%S %p"), 'utf8')
        submission_name = job_preset.get_job_preset_attribute_by_key(api_constants.API_KEYS.SUBMISSION_NAME).eval(self.get_plugin(), None)

        # use a timestamp if no submission name is provided
        if not submission_name:
            submission_name = submission_name_timestamp

        path_safe_submission_name = submission_name.replace(' ', '_').replace(':', '-').replace('.', '_')

        remote_output_path = "/{0}/render_results/{1}".format(project_name, path_safe_submission_name)
        attributes[api_constants.API_KEYS.REMOTE_OUTPUT_FOLDER_PATH] = remote_output_path

        # set watch files
        remote_watch_path = "{0}/.+".format(remote_output_path)

        local_download_path_attribute = job_preset.get_job_preset_attribute_by_key(
            api_constants.API_KEYS.LOCAL_DOWNLOAD_PATH)
        local_download_path = local_download_path_attribute.eval(self.get_plugin(), None)
        if not local_download_path:
            local_download_path = "{0}/{1}/render_results/{2}".format(gm_meta_plugin.utils.get_desktop_path(),
                                                                      project_name,
                                                                      path_safe_submission_name)

        attributes[api_constants.API_KEYS.WATCH_FILES] = "{0},{1}".format(remote_watch_path, local_download_path)

        packed_project = gm_meta_plugin.PackedProject(project_name, root_dir, project_files, attributes)

        return packed_project

    def get_project_type_id(self):
        return api_constants.PROJECT_TYPE_IDS.MAYA_PROJECT

    def get_product_type(self):
        application_attribute_source = self.get_plugin().get_application_attribute_source()
        return application_attribute_source.get_attribute_value(None, None, api_constants.API_KEYS.APP)

    def get_product_version(self):
        application_attribute_source = self.get_plugin().get_application_attribute_source()
        return application_attribute_source.get_attribute_value(None, None, api_constants.API_KEYS.APP_VERSION)
